/*
 *
 *  * Copyright (C) 2014 Numatica Corporation.
 *  * All Rights Reserved.
 *  *
 *  * NOTICE:  All information contained herein is, and remains
 *  * the property of Numatica Corporation and its suppliers,
 *  * if any.  The intellectual and technical concepts contained
 *  * herein are proprietary to Numatica Corporation
 *  * and its suppliers and may be covered by U.S. and Foreign Patents,
 *  * patents in process, and are protected by trade secret or copyright law.
 *  * Dissemination of this information or reproduction of this material
 *  * is strictly forbidden unless prior written permission is obtained
 *  * from Numatica Corporation.
 *
 */
package com.numatica.insightmagic.geolocation.store;

import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.hazelcast.core.MapStore;
import com.numatica.insightmagic.external.ApiException;
import com.numatica.insightmagic.geolocation.model.GeoLocation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.PreparedStatementCallback;
import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcDaoSupport;
import org.springframework.stereotype.Repository;

import javax.sql.DataSource;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

/**
 * Created with IntelliJ IDEA.
 * User: Jacek
 * Date: 10/3/14
 * Time: 1:06 PM
 * To change this template use File | Settings | File Templates.
 */
@Repository("geoStore")
public class GeoStore extends NamedParameterJdbcDaoSupport implements MapStore<Long, GeoLocation> {

    private static final String SQL_LOAD_ALL = "SELECT ipStartNum FROM GeoLocation";
    private static final String SQL_LOAD_ONE = "SELECT *  FROM GeoLocation g  WHERE g.ipStartNum = ?";
    private static final String UPDATE_SQL = "update GeoLocation set " +
            " ipEndNum = :ipEndNum, ipStart = :ipStart, ipEnd = :ipEnd, countryCode = :countryCode, " +
            " stateOrProvince = :stateOrProvince, city = :city " +
            " where ipStartNum = :ipStartNum";
    private static final String INSERT_SQL = "insert into GeoLocation(ipStartNum, ipStart, ipEndNum, ipEnd, " +
            " countryCode, stateOrProvince, city) " +
            " values(:ipStartNum, :ipStart, :ipEndNum, :ipEnd, :countryCode, :stateOrProvince, :city) ";

    protected final Logger log = LoggerFactory.getLogger(getClass());

    private ResultSetExtractor rse = new ResultSetExtractor<GeoLocation>() {
        @Override
        public GeoLocation extractData(ResultSet resultSet) throws SQLException, DataAccessException {
            GeoLocation geoLocation = null;

            while (resultSet.next()) {
                if (geoLocation == null) {
                    geoLocation = new GeoLocation();
                    geoLocation.setIpStartNum(resultSet.getLong("ipStartNum"));
                    geoLocation.setIpEndNum(resultSet.getLong("ipEndNum"));
                    geoLocation.setIpStart(resultSet.getString("ipStart"));
                    geoLocation.setIpEnd(resultSet.getString("ipEnd"));
                    geoLocation.setCountryCode(resultSet.getString("countryCode"));
                    geoLocation.setStateOrProvince(resultSet.getString("stateOrProvince"));
                    geoLocation.setCity(resultSet.getString("city"));
                }
            }
            return geoLocation;
        }
    };

    @SuppressWarnings("SpringJavaAutowiringInspection")
    @Autowired
    public GeoStore(final DataSource dataSource) {
        setDataSource(dataSource);
    }

    @Override
    public GeoLocation load(Long id) {
        GeoLocation geoLocation = (GeoLocation) getJdbcTemplate().query(SQL_LOAD_ONE, rse, id);
        return geoLocation;
    }

    @Override
    public Map<Long, GeoLocation> loadAll(Collection<Long> ipStartNums) {
        Map<Long, GeoLocation> geoMap = Maps.newHashMap();
        for (long ipStartNum : ipStartNums) {
            geoMap.put(ipStartNum, load(ipStartNum));
        }
        return geoMap;
    }

    @Override
    public Set<Long> loadAllKeys() {
        List<Long> idList = getJdbcTemplate().queryForList(SQL_LOAD_ALL, Long.class);
        return Sets.newTreeSet(idList);
    }

    @Override
    public void store(Long aLong, GeoLocation geoLocation) {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put("ipStartNum", geoLocation.getIpStartNum());
        parameters.put("ipStart", geoLocation.getIpStart());
        parameters.put("ipEndNum", geoLocation.getIpEndNum());
        parameters.put("ipEnd", geoLocation.getIpEnd());
        parameters.put("countryCode", geoLocation.getCountryCode());
        parameters.put("stateOrProvince", geoLocation.getStateOrProvince());
        parameters.put("city", geoLocation.getCity());

        /**
         * Todo: explore Postgres way to do the upsert as one atomic op
         */
        try {
            MapSqlParameterSource parameterSource = new MapSqlParameterSource(parameters);
            int updated = getNamedParameterJdbcTemplate().update(UPDATE_SQL, parameterSource);
            if (updated != 0) {
                return;
            }
            getNamedParameterJdbcTemplate().execute(INSERT_SQL, parameterSource, new PreparedStatementCallback() {
                @Override
                public Object doInPreparedStatement(PreparedStatement ps) throws SQLException, DataAccessException {
                    return ps.executeUpdate();
                }
            });
        } catch (Exception ex) {
            log.error("Exception in creating a new userSession in DB ", ex);
            throw new ApiException("Failed to persist userSession object in the database");
        }
    }

    @Override
    public void storeAll(Map<Long, GeoLocation> longGeoLocationMap) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    @Override
    public void delete(Long ipStartNum) {
        getJdbcTemplate().update("DELETE FROM GeoLocation WHERE GeoLocation.ipStartNum = ?", new Long[]{ipStartNum});
    }

    @Override
    public void deleteAll(Collection<Long> longs) {
        //To change body of implemented methods use File | Settings | File Templates.
    }
}
