/*
 *
 *  * Copyright (C) 2014 Numatica Corporation.
 *  * All Rights Reserved.
 *  *
 *  * NOTICE:  All information contained herein is, and remains
 *  * the property of Numatica Corporation and its suppliers,
 *  * if any.  The intellectual and technical concepts contained
 *  * herein are proprietary to Numatica Corporation
 *  * and its suppliers and may be covered by U.S. and Foreign Patents,
 *  * patents in process, and are protected by trade secret or copyright law.
 *  * Dissemination of this information or reproduction of this material
 *  * is strictly forbidden unless prior written permission is obtained
 *  * from Numatica Corporation.
 *
 */

package com.numatica.insightmagic.interaction;

import com.google.common.collect.Maps;
import com.hazelcast.core.IMap;
import com.hazelcast.core.IQueue;
import com.hazelcast.query.SqlPredicate;
import com.numatica.insightmagic.HazelcastConfiguration;
import com.numatica.insightmagic.geolocation.model.GeoLocation;
import com.numatica.insightmagic.geolocation.service.GeoService;
import com.numatica.insightmagic.service.HazelcastBasedService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.support.incrementer.PostgreSQLSequenceMaxValueIncrementer;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.Collection;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Created by Jacek on 4/21/2014.
 */
@Service
public class UserActionService extends HazelcastBasedService {

    protected final Logger log = LoggerFactory.getLogger(getClass());

    @Autowired
    private GeoService geoService;

    @Autowired
    private PostgreSQLSequenceMaxValueIncrementer userActionSequence;

    IMap<Long, UserAction> userActionHistoryMap;

    @PostConstruct
    private void init() {
        userActionHistoryMap = hz.getMap(HazelcastConfiguration.USER_ACTION_HISTORY_MAP);
    }
    public Long generateUserActionId() {
        return userActionSequence.nextLongValue();
    }

    public boolean registerUserAction(UserAction userAction) {
        IQueue<UserAction> clickStreamQueue = hz.getQueue(HazelcastConfiguration.USER_ACTION_QUEUE);
        boolean offered = clickStreamQueue.offer(userAction);
        log.info("ClickStream Queue size now: " + offered + " " + clickStreamQueue.size());
        return offered;
    }

    public UserAction resolveLocation(UserAction ua) {
        GeoLocation geoLocation = geoService.getGeoLocationByIpAddress(ua.getIpAddress());
        if (geoLocation == null) {
            ua.setCity("Unknown");
            ua.setCountryCode("??");
            ua.setStateOrProvince("??");
            return ua;
        }

        ua.setStateOrProvince(geoLocation.getStateOrProvince());
        ua.setCountryCode(geoLocation.getCountryCode());
        ua.setCity(geoLocation.getCity());
        return ua;
    }

    public Map<String, Long> getUserActions(String predicate) {

        Collection<UserAction> actions = userActionHistoryMap.values(new SqlPredicate(predicate));
        Map<String, Long> byCountry;
        if (actions != null) {
            byCountry = actions.stream()
                .collect(Collectors.groupingBy(UserAction::getCountryCode, Collectors.counting()));
        } else {
            byCountry = Maps.newHashMap();
        }

        return byCountry;
    }
}