

/*
 *
 *  * Copyright (C) 2014 Numatica Corporation.
 *  * All Rights Reserved.
 *  *
 *  * NOTICE:  All information contained herein is, and remains
 *  * the property of Numatica Corporation and its suppliers,
 *  * if any.  The intellectual and technical concepts contained
 *  * herein are proprietary to Numatica Corporation
 *  * and its suppliers and may be covered by U.S. and Foreign Patents,
 *  * patents in process, and are protected by trade secret or copyright law.
 *  * Dissemination of this information or reproduction of this material
 *  * is strictly forbidden unless prior written permission is obtained
 *  * from Numatica Corporation.
 *
 */

package com.numatica.insightmagic.interaction;

import com.numatica.insightmagic.authentication.AuthenticatedCall;
import com.numatica.insightmagic.external.ApiReturnCode;
import com.numatica.insightmagic.external.ResponseStatus;
import com.numatica.insightmagic.external.RestResponse;
import com.numatica.insightmagic.interaction.external.UserActionResponse;
import com.numatica.insightmagic.session.service.SessionService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
 * Created by Jacek on 4/21/2014.
 */
@RestController
@RequestMapping("/useraction")
public class UserActionServiceController {

    protected final Logger log = LoggerFactory.getLogger(UserActionServiceController.class);
    @Autowired
    private UserActionService userActionService;
    @Autowired
    private SessionService sessionService;

    @RequestMapping("/registerUserAction")
    @ResponseBody
    @AuthenticatedCall(userSessionRequired = false)
    public RestResponse registerUserAction(Long apiKey, Long sessionKey, String actionCode, String ipAddress, Long timestamp) {

        ResponseStatus status;

        if ((status = sessionService.checkTextArgument(actionCode, "actionCode")) != null) {
            return new RestResponse(status);
        }

        if (ipAddress == null) {
            return new RestResponse(new ResponseStatus(ApiReturnCode.INVALID_ARGUMENT, "IP Address required"));
        }

        if (timestamp == null) {
            return new RestResponse(new ResponseStatus(ApiReturnCode.INVALID_ARGUMENT, "Timestamp required"));
        }


        UserAction userAction = new UserAction();

        if (sessionKey != null) {
            userAction.setUserSessionId(sessionKey);
            userAction.setUserId(sessionService.getSession(sessionKey).getUserId());
        }

        userAction.setActionCode(actionCode);
        userAction.setIpAddress(ipAddress);
        userAction.setTimestamp(timestamp);
        userActionService.registerUserAction(userAction);
        return new RestResponse(new ResponseStatus(ApiReturnCode.SUCCESS));

    }

    @RequestMapping("/getUserActions")
    @ResponseBody
    @AuthenticatedCall(userSessionRequired = false)
    public RestResponse getUserActions(Long apiKey, Long sessionKey, String predicate) {

        ResponseStatus status;

        //if ((status = sessionService.checkTextArgument(predicate, "predicate")) != null) {
        //    return new RestResponse(status);
        //}

        /*if (ipAddress == null) {
            return new RestResponse(new ResponseStatus(ApiReturnCode.INVALID_ARGUMENT, "IP Address required"));
        }

        if (timestamp == null) {
            return new RestResponse(new ResponseStatus(ApiReturnCode.INVALID_ARGUMENT, "Timestamp required"));
        } */

        Map<String, Long> byCountry = userActionService.getUserActions(predicate);
        log.info("By Country, size " + byCountry.size());

        return new UserActionResponse(new ResponseStatus(ApiReturnCode.SUCCESS), byCountry);
    }


}
