/*
 *
 *  * Copyright (C) 2014 Numatica Corporation.
 *  * All Rights Reserved.
 *  *
 *  * NOTICE:  All information contained herein is, and remains
 *  * the property of Numatica Corporation and its suppliers,
 *  * if any.  The intellectual and technical concepts contained
 *  * herein are proprietary to Numatica Corporation
 *  * and its suppliers and may be covered by U.S. and Foreign Patents,
 *  * patents in process, and are protected by trade secret or copyright law.
 *  * Dissemination of this information or reproduction of this material
 *  * is strictly forbidden unless prior written permission is obtained
 *  * from Numatica Corporation.
 *
 */
package com.numatica.insightmagic.queue;

import com.hazelcast.core.HazelcastInstance;
import com.hazelcast.core.IQueue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.Serializable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * Created with IntelliJ IDEA.
 * User: Jacek
 * Date: 10/14/14
 * Time: 4:28 PM
 * To change this template use File | Settings | File Templates.
 */
public abstract class QueueProcessor {
    private static final Logger log = LoggerFactory.getLogger(QueueProcessor.class);
    @Autowired
    protected HazelcastInstance hz;
    private ExecutorService svc;

    public QueueProcessor(int threadCount) {
        try {
            svc = Executors.newFixedThreadPool(threadCount);
        } catch (Throwable ex) {
            log.error("Exception " + ex);
        }
    }

    public void start(int numThreads, String queueName) {
        for (int i = 0; i < numThreads; i++) {
            svc.submit(new Runnable() {

                public void run() {
                    log.info("Starting Consumer for queue {}", queueName);

                    // Set the timeout very large in an attempt to disable it
                    IQueue<Serializable> queue = hz.getQueue(queueName);
                    try {
                        Serializable message;
                        while ((message = queue.take()) != null) {
                            log.info("Message picked up, queue size now = " + queue.size());
                            handleMessage(message);
                        }
                    } catch (Throwable t) {
                        log.info("Exception in QueueProcessor ", t );
                    }
                }
            });
        }
    }

    public abstract void handleMessage(Serializable message);
}
