/*
 *
 *  * Copyright (C) 2014 Numatica Corporation.
 *  * All Rights Reserved.
 *  *
 *  * NOTICE:  All information contained herein is, and remains
 *  * the property of Numatica Corporation and its suppliers,
 *  * if any.  The intellectual and technical concepts contained
 *  * herein are proprietary to Numatica Corporation
 *  * and its suppliers and may be covered by U.S. and Foreign Patents,
 *  * patents in process, and are protected by trade secret or copyright law.
 *  * Dissemination of this information or reproduction of this material
 *  * is strictly forbidden unless prior written permission is obtained
 *  * from Numatica Corporation.
 *
 */

package com.numatica.insightmagic;

import com.numatica.insightmagic.session.external.SessionResponse;
import com.numatica.insightmagic.user.external.UserAccountResponse;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.converter.FormHttpMessageConverter;
import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;


/**
 * Created by Jacek on 4/23/2014.
 */
public class GeoDataLoader {

    private static final String url = "http://localhost:8080";
    private RestTemplate restTemplate = new RestTemplate();
  
    private static final String loadGeoDataUrl = "/geoservice/ingestData";


    public static void main(String[] args) {

        GeoDataLoader geoDataLoader = new GeoDataLoader();
        geoDataLoader.initialize();
        geoDataLoader.loadGeoReferenceData(343847L);

    }

    public void initialize() {
        restTemplate.getMessageConverters().add(new MappingJackson2HttpMessageConverter());
        restTemplate.getMessageConverters().add(new StringHttpMessageConverter());
        restTemplate.getMessageConverters().add(new FormHttpMessageConverter());
    }

    public void loadGeoReferenceData(Long sessionId) {
        MultiValueMap<String, String> map = new LinkedMultiValueMap<String, String>();
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        map.add("apiKey", String.valueOf(API_KEY));
        map.add("sessionKey", String.valueOf(sessionId));
        map.add("fileName", "dbip-city1.csv");

        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<MultiValueMap<String, String>>(map, headers);
        long t1 = System.nanoTime();
        SessionResponse response = restTemplate.postForObject(url + loadGeoDataUrl, request, com.numatica.insightmagic.session.external.SessionResponse.class);
        System.out.println("Time = " + (System.nanoTime() - t1) / 1000000 + " ms");
        if (response != null) {
            if (response.getStatus().getStatusCode() == com.numatica.insightmagic.external.ApiReturnCode.SUCCESS.getCode()) {
                System.out.println("Success");
            } else {
                System.out.println("Logout: " + response.getStatus().getMessage());
            }
        } else {
            System.out.println("Logout: Empty response");
        }
    }

}
